package com.agilecontrol.nea.core.security.pwd;

import com.agilecontrol.nea.core.util.WordsUtil;
import com.agilecontrol.nea.util.Validator;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;

public class PwdStrength {

   private static PwdStrength instance = new PwdStrength();
   private char[] lowerChars = this.getSortedCharArray("abcdefghjkmnpqrstuvwxyz");
   private char[] upperChars = this.getSortedCharArray("ABCDEFGHJKLMNPQRSTUVWXYZ");
   private char[] numberChars = this.getSortedCharArray("0123456789");


   private char[] getSortedCharArray(String s) {
      char[] charArray = s.toCharArray();
      Arrays.sort(charArray);
      return charArray;
   }

   private int getUsageCount(String s, char[] charArray) {
      int count = 0;

      for(int i = 0; i < s.length(); ++i) {
         if(Arrays.binarySearch(charArray, s.charAt(i)) >= 0) {
            ++count;
         }
      }

      return count;
   }

   private boolean isSerial(String s) {
      if(s.length() < 2) {
         return false;
      } else {
         char[] cs = s.toCharArray();
         HashSet set = new HashSet();
         boolean isContinuous = true;

         int i;
         for(i = 0; i < cs.length - 1; ++i) {
            isContinuous = cs[i + 1] == cs[i] + 1;
            if(!isContinuous) {
               break;
            }
         }

         if(isContinuous) {
            return true;
         } else {
            isContinuous = true;

            for(i = 0; i < cs.length - 1; ++i) {
               isContinuous = cs[i + 1] == cs[i] - 1;
               if(!isContinuous) {
                  break;
               }
            }

            if(isContinuous) {
               return true;
            } else {
               for(i = 0; i < cs.length - 1; ++i) {
                  set.add(Character.valueOf(cs[i]));
               }

               return set.size() == 1;
            }
         }
      }
   }

   private List split(String s) {
      ArrayList al = new ArrayList();
      StringBuilder sb = new StringBuilder();
      char[] cs = s.toCharArray();
      sb.append(cs[0]);
      char start = cs[0];

      for(int i = 1; i < cs.length; ++i) {
         char c = cs[i];
         if((start < 97 || start > 122 || c < 97 || c > 122) && (start < 65 || start > 90 || c < 65 || c > 90) && (start < 48 || start > 57 || c < 48 || c > 57) && start != c) {
            al.add(sb);
            sb = new StringBuilder();
            sb.append(c);
            start = c;
         } else {
            sb.append(c);
         }
      }

      al.add(sb);
      return al;
   }

   public static int check(String passwd) {
      return instance._check(passwd);
   }

   private int _check(String passwd) {
      if(Validator.isNull(passwd)) {
         return 1;
      } else if(WordsUtil.isDictionaryWord(passwd)) {
         return 1;
      } else {
         int len = passwd.length();
         if(this.isSerial(passwd)) {
            return 1;
         } else if(this.getUsageCount(passwd, this.lowerChars) != len && this.getUsageCount(passwd, this.upperChars) != len && this.getUsageCount(passwd, this.numberChars) != len) {
            List groups = this.split(passwd);
            boolean isAllSerial = true;

            for(int i = 0; i < groups.size(); ++i) {
               if(!this.isSerial(((StringBuilder)groups.get(i)).toString())) {
                  isAllSerial = false;
               }
            }

            if(isAllSerial) {
               return 3;
            } else {
               return 4;
            }
         } else {
            return 2;
         }
      }
   }

   public static void main(String[] args) throws Exception {
      System.out.println(check("1111") == 1);
      System.out.println(check("12345") == 1);
      System.out.println(check("gfed") == 1);
      System.out.println(check("aaabcafdfsdf") == 2);
      System.out.println(check("abc123") == 3);
      System.out.println(check("abc__") == 3);
      System.out.println(check("aaa222") == 3);
      System.out.println(check("aaa222D") == 4);
      System.out.println(check("aaa222DD") == 3);
   }

}
